@@ -13,6 +13,9 @@
  * Copyright (C) 2022 Sebastian Reimers
  */
 
+#ifndef RE_H_THREAD__
+#define RE_H_THREAD__
+
 #if defined(HAVE_THREADS)
 #include <threads.h>
 
@@ -20,6 +23,9 @@
 
 #if defined(WIN32)
 
+#ifndef WIN32_LEAN_AND_MEAN
+#define WIN32_LEAN_AND_MEAN
+#endif
 #include <windows.h>
 #define ONCE_FLAG_INIT INIT_ONCE_STATIC_INIT
 typedef INIT_ONCE once_flag;
@@ -67,7 +73,7 @@ typedef int (*thrd_start_t)(void *);
  * @param func  Function to execute
  * @param arg   Argument to pass to the function
  *
- * @return 0 if success, otherwise errorcode
+ * @return thrd_success on success, otherwise thrd_error
  */
 int thrd_create(thrd_t *thr, thrd_start_t func, void *arg);
 
@@ -96,7 +102,7 @@ thrd_t thrd_current(void);
  *
  * @param thr  Thread
  *
- * @return 0 if success, otherwise errorcode
+ * @return thrd_success on success, otherwise thrd_error
  */
 int thrd_detach(thrd_t thr);
 
@@ -108,7 +114,7 @@ int thrd_detach(thrd_t thr);
  * @param thr  Thread
  * @param res  Result code location
  *
- * @return 0 if success, otherwise errorcode
+ * @return thrd_success on success, otherwise thrd_error
  */
 int thrd_join(thrd_t thr, int *res);
 
@@ -139,7 +145,7 @@ void thrd_exit(int res);
  *
  * @param cnd  Pointer to a variable to store condition variable
  *
- * @return 0 if success, otherwise errorcode
+ * @return thrd_success on success, otherwise thrd_error
  */
 int cnd_init(cnd_t *cnd);
 
@@ -149,7 +155,7 @@ int cnd_init(cnd_t *cnd);
  *
  * @param cnd  Pointer to condition variable
  *
- * @return 0 if success, otherwise errorcode
+ * @return thrd_success on success, otherwise thrd_error
  */
 int cnd_signal(cnd_t *cnd);
 
@@ -159,7 +165,7 @@ int cnd_signal(cnd_t *cnd);
  *
  * @param cnd  Pointer to condition variable
  *
- * @return 0 if success, otherwise errorcode
+ * @return thrd_success on success, otherwise thrd_error
  */
 int cnd_broadcast(cnd_t *cnd);
 
@@ -170,11 +176,24 @@ int cnd_broadcast(cnd_t *cnd);
  * @param cnd   Pointer to condition variable
  * @param mtx   Lock mutex pointer
  *
- * @return 0 if success, otherwise errorcode
+ * @return thrd_success on success, otherwise thrd_error
  */
 int cnd_wait(cnd_t *cnd, mtx_t *mtx);
 
 
+/**
+ * Blocks on a condition variable with timeout (TIME_UTC based)
+ *
+ * @param cnd     Pointer to condition variable
+ * @param mtx     Lock mutex pointer
+ * @param abstime Pointer to timeout time
+ *
+ * @return thrd_success on success, thrd_timedout if the timeout time
+ * has been reached before the mutex is locked, otherwise thrd_error
+ */
+int cnd_timedwait(cnd_t *cnd, mtx_t *mtx, const struct timespec *abstime);
+
+
 /**
  * Destroys the condition variable pointed to by cnd.
  * If there are thrds waiting on cnd, the behavior is undefined.
@@ -256,9 +275,13 @@ void tss_delete(tss_t key);
 
 /******************************************************************************
  * Extra - non C11 helpers
+ * (We avoid tss_ mtx_ cnd_ prefixes since these reserved for functions with
+ * different return values)
  *****************************************************************************/
-/* int thrd_prio(enum thrd_prio prio) */
-/* void thrd_print(struct re_printf *pf, void *unused); */
+
+/* Ideas: */
+/* int thread_prio(enum thrd_prio prio) */
+/* void thread_print(struct re_printf *pf, void *unused); */
 
 /**
  * Allocates and initializes a new mutex
@@ -267,7 +290,7 @@ void tss_delete(tss_t key);
  *
  * @return 0 if success, otherwise errorcode
  */
-int mtx_alloc(mtx_t **mtx);
+int mutex_alloc(mtx_t **mtx);
 
 
 /**
@@ -280,5 +303,7 @@ int mtx_alloc(mtx_t **mtx);
  *
  * @return 0 if success, otherwise errorcode
  */
-int thrd_create_name(thrd_t *thr, const char *name, thrd_start_t func,
+int thread_create_name(thrd_t *thr, const char *name, thrd_start_t func,
 		     void *arg);
+
+#endif /* RE_H_THREAD__ */
